<?php

return [
    'Names' => [
        'Africa/Abidjan' => 'universala tempo kunordigita (Abidjan)',
        'Africa/Accra' => 'universala tempo kunordigita (Accra)',
        'Africa/Addis_Ababa' => 'tempo de Etiopujo (Addis Ababa)',
        'Africa/Algiers' => 'tempo de Alĝerio (Algiers)',
        'Africa/Asmera' => 'tempo de Eritreo (Asmara)',
        'Africa/Bamako' => 'universala tempo kunordigita (Bamako)',
        'Africa/Bangui' => 'tempo de Centr-Afrika Respubliko (Bangui)',
        'Africa/Banjul' => 'universala tempo kunordigita (Banjul)',
        'Africa/Bissau' => 'universala tempo kunordigita (Bissau)',
        'Africa/Blantyre' => 'tempo de Malavio (Blantyre)',
        'Africa/Brazzaville' => 'tempo de Kongo Brazavila (Brazzaville)',
        'Africa/Bujumbura' => 'tempo de Burundo (Bujumbura)',
        'Africa/Cairo' => 'tempo de Egiptujo (Cairo)',
        'Africa/Casablanca' => 'tempo de Maroko (Casablanca)',
        'Africa/Ceuta' => 'tempo de Hispanujo (Ceuta)',
        'Africa/Conakry' => 'universala tempo kunordigita (Conakry)',
        'Africa/Dakar' => 'universala tempo kunordigita (Dakar)',
        'Africa/Dar_es_Salaam' => 'tempo de Tanzanio (Dar es Salaam)',
        'Africa/Djibouti' => 'tempo de Ĝibutio (Djibouti)',
        'Africa/Douala' => 'tempo de Kameruno (Douala)',
        'Africa/El_Aaiun' => 'tempo de Okcidenta Saharo (El Aaiun)',
        'Africa/Freetown' => 'universala tempo kunordigita (Freetown)',
        'Africa/Gaborone' => 'tempo de Bocvano (Gaborone)',
        'Africa/Harare' => 'tempo de Zimbabvo (Harare)',
        'Africa/Johannesburg' => 'tempo de Sud-Afriko (Johannesburg)',
        'Africa/Kampala' => 'tempo de Ugando (Kampala)',
        'Africa/Khartoum' => 'tempo de Sudano (Khartoum)',
        'Africa/Kigali' => 'tempo de Ruando (Kigali)',
        'Africa/Lagos' => 'tempo de Niĝerio (Lagos)',
        'Africa/Libreville' => 'tempo de Gabono (Libreville)',
        'Africa/Lome' => 'universala tempo kunordigita (Lome)',
        'Africa/Luanda' => 'tempo de Angolo (Luanda)',
        'Africa/Lusaka' => 'tempo de Zambio (Lusaka)',
        'Africa/Malabo' => 'tempo de Ekvatora Gvineo (Malabo)',
        'Africa/Maputo' => 'tempo de Mozambiko (Maputo)',
        'Africa/Maseru' => 'tempo de Lesoto (Maseru)',
        'Africa/Mbabane' => 'tempo de Svazilando (Mbabane)',
        'Africa/Mogadishu' => 'tempo de Somalujo (Mogadishu)',
        'Africa/Monrovia' => 'universala tempo kunordigita (Monrovia)',
        'Africa/Nairobi' => 'tempo de Kenjo (Nairobi)',
        'Africa/Ndjamena' => 'tempo de Ĉado (Ndjamena)',
        'Africa/Niamey' => 'tempo de Niĝero (Niamey)',
        'Africa/Nouakchott' => 'universala tempo kunordigita (Nouakchott)',
        'Africa/Ouagadougou' => 'universala tempo kunordigita (Ouagadougou)',
        'Africa/Porto-Novo' => 'tempo de Benino (Porto-Novo)',
        'Africa/Sao_Tome' => 'universala tempo kunordigita (São Tomé)',
        'Africa/Tripoli' => 'tempo de Libio (Tripoli)',
        'Africa/Tunis' => 'tempo de Tunizio (Tunis)',
        'Africa/Windhoek' => 'tempo de Namibio (Windhoek)',
        'America/Adak' => 'tempo de Usono (Adak)',
        'America/Anchorage' => 'tempo de Usono (Anchorage)',
        'America/Anguilla' => 'tempo de Angvilo (Anguilla)',
        'America/Antigua' => 'tempo de Antigvo kaj Barbudo (Antigua)',
        'America/Araguaina' => 'tempo de Brazilo (Araguaina)',
        'America/Argentina/La_Rioja' => 'tempo de Argentino (La Rioja)',
        'America/Argentina/Rio_Gallegos' => 'tempo de Argentino (Rio Gallegos)',
        'America/Argentina/Salta' => 'tempo de Argentino (Salta)',
        'America/Argentina/San_Juan' => 'tempo de Argentino (San Juan)',
        'America/Argentina/San_Luis' => 'tempo de Argentino (San Luis)',
        'America/Argentina/Tucuman' => 'tempo de Argentino (Tucuman)',
        'America/Argentina/Ushuaia' => 'tempo de Argentino (Ushuaia)',
        'America/Aruba' => 'tempo de Arubo (Aruba)',
        'America/Asuncion' => 'tempo de Paragvajo (Asunción)',
        'America/Bahia' => 'tempo de Brazilo (Bahia)',
        'America/Bahia_Banderas' => 'tempo de Meksiko (Bahía de Banderas)',
        'America/Barbados' => 'tempo de Barbado (Barbados)',
        'America/Belem' => 'tempo de Brazilo (Belem)',
        'America/Belize' => 'tempo de Belizo (Belize)',
        'America/Blanc-Sablon' => 'tempo de Kanado (Blanc-Sablon)',
        'America/Boa_Vista' => 'tempo de Brazilo (Boa Vista)',
        'America/Bogota' => 'tempo de Kolombio (Bogota)',
        'America/Boise' => 'tempo de Usono (Boise)',
        'America/Buenos_Aires' => 'tempo de Argentino (Buenos Aires)',
        'America/Cambridge_Bay' => 'tempo de Kanado (Cambridge Bay)',
        'America/Campo_Grande' => 'tempo de Brazilo (Campo Grande)',
        'America/Cancun' => 'tempo de Meksiko (Cancún)',
        'America/Caracas' => 'tempo de Venezuelo (Caracas)',
        'America/Catamarca' => 'tempo de Argentino (Catamarca)',
        'America/Cayenne' => 'tempo de Franca Gviano (Cayenne)',
        'America/Cayman' => 'tempo de Kejmanoj (Cayman)',
        'America/Chicago' => 'tempo de Usono (Chicago)',
        'America/Chihuahua' => 'tempo de Meksiko (Chihuahua)',
        'America/Ciudad_Juarez' => 'tempo de Meksiko (Ciudad Juárez)',
        'America/Coral_Harbour' => 'tempo de Kanado (Atikokan)',
        'America/Cordoba' => 'tempo de Argentino (Cordoba)',
        'America/Costa_Rica' => 'tempo de Kostariko (Costa Rica)',
        'America/Creston' => 'tempo de Kanado (Creston)',
        'America/Cuiaba' => 'tempo de Brazilo (Cuiaba)',
        'America/Danmarkshavn' => 'universala tempo kunordigita (Danmarkshavn)',
        'America/Dawson' => 'tempo de Kanado (Dawson)',
        'America/Dawson_Creek' => 'tempo de Kanado (Dawson Creek)',
        'America/Denver' => 'tempo de Usono (Denver)',
        'America/Detroit' => 'tempo de Usono (Detroit)',
        'America/Dominica' => 'tempo de Dominiko (Dominica)',
        'America/Edmonton' => 'tempo de Kanado (Edmonton)',
        'America/Eirunepe' => 'tempo de Brazilo (Eirunepe)',
        'America/El_Salvador' => 'tempo de Salvadoro (El Salvador)',
        'America/Fort_Nelson' => 'tempo de Kanado (Fort Nelson)',
        'America/Fortaleza' => 'tempo de Brazilo (Fortaleza)',
        'America/Glace_Bay' => 'tempo de Kanado (Glace Bay)',
        'America/Godthab' => 'tempo de Gronlando (Nuuk)',
        'America/Goose_Bay' => 'tempo de Kanado (Goose Bay)',
        'America/Grenada' => 'tempo de Grenado (Grenada)',
        'America/Guadeloupe' => 'tempo de Gvadelupo (Guadeloupe)',
        'America/Guatemala' => 'tempo de Gvatemalo (Guatemala)',
        'America/Guayaquil' => 'tempo de Ekvadoro (Guayaquil)',
        'America/Guyana' => 'tempo de Gujano (Guyana)',
        'America/Halifax' => 'tempo de Kanado (Halifax)',
        'America/Havana' => 'tempo de Kubo (Havana)',
        'America/Hermosillo' => 'tempo de Meksiko (Hermosillo)',
        'America/Indiana/Knox' => 'tempo de Usono (Knox, Indiana)',
        'America/Indiana/Marengo' => 'tempo de Usono (Marengo, Indiana)',
        'America/Indiana/Petersburg' => 'tempo de Usono (Petersburg, Indiana)',
        'America/Indiana/Tell_City' => 'tempo de Usono (Tell City, Indiana)',
        'America/Indiana/Vevay' => 'tempo de Usono (Vevay, Indiana)',
        'America/Indiana/Vincennes' => 'tempo de Usono (Vincennes, Indiana)',
        'America/Indiana/Winamac' => 'tempo de Usono (Winamac, Indiana)',
        'America/Indianapolis' => 'tempo de Usono (Indianapolis)',
        'America/Inuvik' => 'tempo de Kanado (Inuvik)',
        'America/Iqaluit' => 'tempo de Kanado (Iqaluit)',
        'America/Jamaica' => 'tempo de Jamajko (Jamaica)',
        'America/Jujuy' => 'tempo de Argentino (Jujuy)',
        'America/Juneau' => 'tempo de Usono (Juneau)',
        'America/Kentucky/Monticello' => 'tempo de Usono (Monticello, Kentucky)',
        'America/La_Paz' => 'tempo de Bolivio (La Paz)',
        'America/Lima' => 'tempo de Peruo (Lima)',
        'America/Los_Angeles' => 'tempo de Usono (Los Angeles)',
        'America/Louisville' => 'tempo de Usono (Louisville)',
        'America/Maceio' => 'tempo de Brazilo (Maceio)',
        'America/Managua' => 'tempo de Nikaragvo (Managua)',
        'America/Manaus' => 'tempo de Brazilo (Manaus)',
        'America/Martinique' => 'tempo de Martiniko (Martinique)',
        'America/Matamoros' => 'tempo de Meksiko (Matamoros)',
        'America/Mazatlan' => 'tempo de Meksiko (Mazatlan)',
        'America/Mendoza' => 'tempo de Argentino (Mendoza)',
        'America/Menominee' => 'tempo de Usono (Menominee)',
        'America/Merida' => 'tempo de Meksiko (Mérida)',
        'America/Metlakatla' => 'tempo de Usono (Metlakatla)',
        'America/Mexico_City' => 'tempo de Meksiko (Mexico City)',
        'America/Miquelon' => 'tempo de Sankta Piero kaj Mikelono (Miquelon)',
        'America/Moncton' => 'tempo de Kanado (Moncton)',
        'America/Monterrey' => 'tempo de Meksiko (Monterrey)',
        'America/Montevideo' => 'tempo de Urugvajo (Montevideo)',
        'America/Nassau' => 'tempo de Bahamoj (Nassau)',
        'America/New_York' => 'tempo de Usono (New York)',
        'America/Nome' => 'tempo de Usono (Nome)',
        'America/Noronha' => 'tempo de Brazilo (Noronha)',
        'America/North_Dakota/Beulah' => 'tempo de Usono (Beulah, North Dakota)',
        'America/North_Dakota/Center' => 'tempo de Usono (Center, North Dakota)',
        'America/North_Dakota/New_Salem' => 'tempo de Usono (New Salem, North Dakota)',
        'America/Ojinaga' => 'tempo de Meksiko (Ojinaga)',
        'America/Panama' => 'tempo de Panamo (Panama)',
        'America/Paramaribo' => 'tempo de Surinamo (Paramaribo)',
        'America/Phoenix' => 'tempo de Usono (Phoenix)',
        'America/Port-au-Prince' => 'tempo de Haitio (Port-au-Prince)',
        'America/Port_of_Spain' => 'tempo de Trinidado kaj Tobago (Port of Spain)',
        'America/Porto_Velho' => 'tempo de Brazilo (Porto Velho)',
        'America/Puerto_Rico' => 'tempo de Puertoriko (Puerto Rico)',
        'America/Punta_Arenas' => 'tempo de Ĉilio (Punta Arenas)',
        'America/Rankin_Inlet' => 'tempo de Kanado (Rankin Inlet)',
        'America/Recife' => 'tempo de Brazilo (Recife)',
        'America/Regina' => 'tempo de Kanado (Regina)',
        'America/Resolute' => 'tempo de Kanado (Resolute)',
        'America/Rio_Branco' => 'tempo de Brazilo (Rio Branco)',
        'America/Santarem' => 'tempo de Brazilo (Santarem)',
        'America/Santiago' => 'tempo de Ĉilio (Santiago)',
        'America/Santo_Domingo' => 'tempo de Domingo (Santo Domingo)',
        'America/Sao_Paulo' => 'tempo de Brazilo (Sao Paulo)',
        'America/Scoresbysund' => 'tempo de Gronlando (Ittoqqortoormiit)',
        'America/Sitka' => 'tempo de Usono (Sitka)',
        'America/St_Johns' => 'tempo de Kanado (St. John’s)',
        'America/St_Kitts' => 'tempo de Sankta Kristoforo kaj Neviso (St. Kitts)',
        'America/St_Lucia' => 'tempo de Sankta Lucio (St. Lucia)',
        'America/St_Thomas' => 'tempo de Usonaj Virgulininsuloj (St. Thomas)',
        'America/St_Vincent' => 'tempo de Sankta Vincento kaj Grenadinoj (St. Vincent)',
        'America/Swift_Current' => 'tempo de Kanado (Swift Current)',
        'America/Tegucigalpa' => 'tempo de Honduro (Tegucigalpa)',
        'America/Thule' => 'tempo de Gronlando (Thule)',
        'America/Tijuana' => 'tempo de Meksiko (Tijuana)',
        'America/Toronto' => 'tempo de Kanado (Toronto)',
        'America/Tortola' => 'tempo de Britaj Virgulininsuloj (Tortola)',
        'America/Vancouver' => 'tempo de Kanado (Vancouver)',
        'America/Whitehorse' => 'tempo de Kanado (Whitehorse)',
        'America/Winnipeg' => 'tempo de Kanado (Winnipeg)',
        'America/Yakutat' => 'tempo de Usono (Yakutat)',
        'Antarctica/Casey' => 'tempo de Antarkto (Casey)',
        'Antarctica/Davis' => 'tempo de Antarkto (Davis)',
        'Antarctica/DumontDUrville' => 'tempo de Antarkto (Dumont d’Urville)',
        'Antarctica/Macquarie' => 'tempo de Aŭstralio (Macquarie)',
        'Antarctica/Mawson' => 'tempo de Antarkto (Mawson)',
        'Antarctica/McMurdo' => 'tempo de Antarkto (McMurdo)',
        'Antarctica/Palmer' => 'tempo de Antarkto (Palmer)',
        'Antarctica/Rothera' => 'tempo de Antarkto (Rothera)',
        'Antarctica/Syowa' => 'tempo de Antarkto (Syowa)',
        'Antarctica/Troll' => 'universala tempo kunordigita (Troll)',
        'Antarctica/Vostok' => 'tempo de Antarkto (Vostok)',
        'Arctic/Longyearbyen' => 'tempo de Svalbardo kaj Janmajeno (Longyearbyen)',
        'Asia/Aden' => 'tempo de Jemeno (Aden)',
        'Asia/Almaty' => 'tempo de Kazaĥujo (Almaty)',
        'Asia/Amman' => 'tempo de Jordanio (Amman)',
        'Asia/Anadyr' => 'tempo de Rusujo (Anadyr)',
        'Asia/Aqtau' => 'tempo de Kazaĥujo (Aqtau)',
        'Asia/Aqtobe' => 'tempo de Kazaĥujo (Aqtobe)',
        'Asia/Ashgabat' => 'tempo de Turkmenujo (Ashgabat)',
        'Asia/Atyrau' => 'tempo de Kazaĥujo (Atyrau)',
        'Asia/Baghdad' => 'tempo de Irako (Baghdad)',
        'Asia/Bahrain' => 'tempo de Barejno (Bahrain)',
        'Asia/Baku' => 'tempo de Azerbajĝano (Baku)',
        'Asia/Bangkok' => 'tempo de Tajlando (Bangkok)',
        'Asia/Barnaul' => 'tempo de Rusujo (Barnaul)',
        'Asia/Beirut' => 'tempo de Libano (Beirut)',
        'Asia/Bishkek' => 'tempo de Kirgizujo (Bishkek)',
        'Asia/Brunei' => 'tempo de Brunejo (Brunei)',
        'Asia/Calcutta' => 'tempo de Hindujo (Kolkata)',
        'Asia/Chita' => 'tempo de Rusujo (Chita)',
        'Asia/Colombo' => 'tempo de Srilanko (Colombo)',
        'Asia/Damascus' => 'tempo de Sirio (Damascus)',
        'Asia/Dhaka' => 'tempo de Bangladeŝo (Dhaka)',
        'Asia/Dubai' => 'tempo de Unuiĝintaj Arabaj Emirlandoj (Dubai)',
        'Asia/Dushanbe' => 'tempo de Taĝikujo (Dushanbe)',
        'Asia/Famagusta' => 'tempo de Kipro (Famagusta)',
        'Asia/Hovd' => 'tempo de Mongolujo (Hovd)',
        'Asia/Irkutsk' => 'tempo de Rusujo (Irkutsk)',
        'Asia/Jakarta' => 'tempo de Indonezio (Jakarta)',
        'Asia/Jayapura' => 'tempo de Indonezio (Jayapura)',
        'Asia/Jerusalem' => 'tempo de Israelo (Jerusalem)',
        'Asia/Kabul' => 'tempo de Afganujo (Kabul)',
        'Asia/Kamchatka' => 'tempo de Rusujo (Kamchatka)',
        'Asia/Karachi' => 'tempo de Pakistano (Karachi)',
        'Asia/Katmandu' => 'tempo de Nepalo (Kathmandu)',
        'Asia/Khandyga' => 'tempo de Rusujo (Khandyga)',
        'Asia/Krasnoyarsk' => 'tempo de Rusujo (Krasnoyarsk)',
        'Asia/Kuala_Lumpur' => 'tempo de Malajzio (Kuala Lumpur)',
        'Asia/Kuching' => 'tempo de Malajzio (Kuching)',
        'Asia/Kuwait' => 'tempo de Kuvajto (Kuwait)',
        'Asia/Magadan' => 'tempo de Rusujo (Magadan)',
        'Asia/Makassar' => 'tempo de Indonezio (Makassar)',
        'Asia/Manila' => 'tempo de Filipinoj (Manila)',
        'Asia/Muscat' => 'tempo de Omano (Muscat)',
        'Asia/Nicosia' => 'tempo de Kipro (Nicosia)',
        'Asia/Novokuznetsk' => 'tempo de Rusujo (Novokuznetsk)',
        'Asia/Novosibirsk' => 'tempo de Rusujo (Novosibirsk)',
        'Asia/Omsk' => 'tempo de Rusujo (Omsk)',
        'Asia/Oral' => 'tempo de Kazaĥujo (Oral)',
        'Asia/Phnom_Penh' => 'tempo de Kamboĝo (Phnom Penh)',
        'Asia/Pontianak' => 'tempo de Indonezio (Pontianak)',
        'Asia/Pyongyang' => 'tempo de Nord-Koreo (Pyongyang)',
        'Asia/Qatar' => 'tempo de Kataro (Qatar)',
        'Asia/Qostanay' => 'tempo de Kazaĥujo (Qostanay)',
        'Asia/Qyzylorda' => 'tempo de Kazaĥujo (Qyzylorda)',
        'Asia/Rangoon' => 'tempo de Birmo (Yangon)',
        'Asia/Riyadh' => 'tempo de Sauda Arabujo (Riyadh)',
        'Asia/Saigon' => 'tempo de Vjetnamo (Ho Chi Minh)',
        'Asia/Sakhalin' => 'tempo de Rusujo (Sakhalin)',
        'Asia/Samarkand' => 'tempo de Uzbekujo (Samarkand)',
        'Asia/Seoul' => 'tempo de Sud-Koreo (Seoul)',
        'Asia/Shanghai' => 'tempo de Ĉinujo (Shanghai)',
        'Asia/Singapore' => 'tempo de Singapuro (Singapore)',
        'Asia/Srednekolymsk' => 'tempo de Rusujo (Srednekolymsk)',
        'Asia/Taipei' => 'tempo de Tajvano (Taipei)',
        'Asia/Tashkent' => 'tempo de Uzbekujo (Tashkent)',
        'Asia/Tbilisi' => 'tempo de Kartvelujo (Tbilisi)',
        'Asia/Tehran' => 'tempo de Irano (Tehran)',
        'Asia/Thimphu' => 'tempo de Butano (Thimphu)',
        'Asia/Tokyo' => 'tempo de Japanujo (Tokyo)',
        'Asia/Tomsk' => 'tempo de Rusujo (Tomsk)',
        'Asia/Ulaanbaatar' => 'tempo de Mongolujo (Ulaanbaatar)',
        'Asia/Urumqi' => 'tempo de Ĉinujo (Urumqi)',
        'Asia/Ust-Nera' => 'tempo de Rusujo (Ust-Nera)',
        'Asia/Vientiane' => 'tempo de Laoso (Vientiane)',
        'Asia/Vladivostok' => 'tempo de Rusujo (Vladivostok)',
        'Asia/Yakutsk' => 'tempo de Rusujo (Yakutsk)',
        'Asia/Yekaterinburg' => 'tempo de Rusujo (Yekaterinburg)',
        'Asia/Yerevan' => 'tempo de Armenujo (Yerevan)',
        'Atlantic/Azores' => 'tempo de Portugalujo (Azores)',
        'Atlantic/Bermuda' => 'tempo de Bermudoj (Bermuda)',
        'Atlantic/Canary' => 'tempo de Hispanujo (Canary)',
        'Atlantic/Cape_Verde' => 'tempo de Kaboverdo (Cape Verde)',
        'Atlantic/Faeroe' => 'tempo de Ferooj (Faroe)',
        'Atlantic/Madeira' => 'tempo de Portugalujo (Madeira)',
        'Atlantic/Reykjavik' => 'universala tempo kunordigita (Reykjavik)',
        'Atlantic/South_Georgia' => 'tempo de Sud-Georgio kaj Sud-Sandviĉinsuloj (South Georgia)',
        'Atlantic/St_Helena' => 'universala tempo kunordigita (St. Helena)',
        'Australia/Adelaide' => 'tempo de Aŭstralio (Adelaide)',
        'Australia/Brisbane' => 'tempo de Aŭstralio (Brisbane)',
        'Australia/Broken_Hill' => 'tempo de Aŭstralio (Broken Hill)',
        'Australia/Darwin' => 'tempo de Aŭstralio (Darwin)',
        'Australia/Eucla' => 'tempo de Aŭstralio (Eucla)',
        'Australia/Hobart' => 'tempo de Aŭstralio (Hobart)',
        'Australia/Lindeman' => 'tempo de Aŭstralio (Lindeman)',
        'Australia/Lord_Howe' => 'tempo de Aŭstralio (Lord Howe)',
        'Australia/Melbourne' => 'tempo de Aŭstralio (Melbourne)',
        'Australia/Perth' => 'tempo de Aŭstralio (Perth)',
        'Australia/Sydney' => 'tempo de Aŭstralio (Sydney)',
        'Etc/GMT' => 'universala tempo kunordigita',
        'Europe/Amsterdam' => 'tempo de Nederlando (Amsterdam)',
        'Europe/Andorra' => 'tempo de Andoro (Andorra)',
        'Europe/Astrakhan' => 'tempo de Rusujo (Astrakhan)',
        'Europe/Athens' => 'tempo de Grekujo (Athens)',
        'Europe/Berlin' => 'tempo de Germanujo (Berlin)',
        'Europe/Bratislava' => 'tempo de Slovakujo (Bratislava)',
        'Europe/Brussels' => 'tempo de Belgujo (Brussels)',
        'Europe/Bucharest' => 'tempo de Rumanujo (Bucharest)',
        'Europe/Budapest' => 'tempo de Hungarujo (Budapest)',
        'Europe/Busingen' => 'tempo de Germanujo (Busingen)',
        'Europe/Chisinau' => 'tempo de Moldavujo (Chisinau)',
        'Europe/Copenhagen' => 'tempo de Danujo (Copenhagen)',
        'Europe/Dublin' => 'universala tempo kunordigita (Dublin)',
        'Europe/Gibraltar' => 'tempo de Ĝibraltaro (Gibraltar)',
        'Europe/Guernsey' => 'universala tempo kunordigita (Guernsey)',
        'Europe/Helsinki' => 'tempo de Finnlando (Helsinki)',
        'Europe/Isle_of_Man' => 'universala tempo kunordigita (Isle of Man)',
        'Europe/Istanbul' => 'tempo de Turkujo (Istanbul)',
        'Europe/Jersey' => 'universala tempo kunordigita (Jersey)',
        'Europe/Kaliningrad' => 'tempo de Rusujo (Kaliningrad)',
        'Europe/Kiev' => 'tempo de Ukrainujo (Kyiv)',
        'Europe/Kirov' => 'tempo de Rusujo (Kirov)',
        'Europe/Lisbon' => 'tempo de Portugalujo (Lisbon)',
        'Europe/Ljubljana' => 'tempo de Slovenujo (Ljubljana)',
        'Europe/London' => 'universala tempo kunordigita (London)',
        'Europe/Luxembourg' => 'tempo de Luksemburgo (Luxembourg)',
        'Europe/Madrid' => 'tempo de Hispanujo (Madrid)',
        'Europe/Malta' => 'tempo de Malto (Malta)',
        'Europe/Minsk' => 'tempo de Belorusujo (Minsk)',
        'Europe/Monaco' => 'tempo de Monako (Monaco)',
        'Europe/Moscow' => 'tempo de Rusujo (Moscow)',
        'Europe/Oslo' => 'tempo de Norvegujo (Oslo)',
        'Europe/Paris' => 'tempo de Francujo (Paris)',
        'Europe/Prague' => 'tempo de Ĉeĥujo (Prague)',
        'Europe/Riga' => 'tempo de Latvujo (Riga)',
        'Europe/Rome' => 'tempo de Italujo (Rome)',
        'Europe/Samara' => 'tempo de Rusujo (Samara)',
        'Europe/San_Marino' => 'tempo de Sanmarino (San Marino)',
        'Europe/Sarajevo' => 'tempo de Bosnujo kaj Hercegovino (Sarajevo)',
        'Europe/Saratov' => 'tempo de Rusujo (Saratov)',
        'Europe/Simferopol' => 'tempo de Ukrainujo (Simferopol)',
        'Europe/Sofia' => 'tempo de Bulgarujo (Sofia)',
        'Europe/Stockholm' => 'tempo de Svedujo (Stockholm)',
        'Europe/Tallinn' => 'tempo de Estonujo (Tallinn)',
        'Europe/Tirane' => 'tempo de Albanujo (Tirane)',
        'Europe/Ulyanovsk' => 'tempo de Rusujo (Ulyanovsk)',
        'Europe/Vaduz' => 'tempo de Liĥtenŝtejno (Vaduz)',
        'Europe/Vatican' => 'tempo de Vatikano (Vatican)',
        'Europe/Vienna' => 'tempo de Aŭstrujo (Vienna)',
        'Europe/Vilnius' => 'tempo de Litovujo (Vilnius)',
        'Europe/Volgograd' => 'tempo de Rusujo (Volgograd)',
        'Europe/Warsaw' => 'tempo de Pollando (Warsaw)',
        'Europe/Zagreb' => 'tempo de Kroatujo (Zagreb)',
        'Europe/Zurich' => 'tempo de Svisujo (Zurich)',
        'Indian/Antananarivo' => 'tempo de Madagaskaro (Antananarivo)',
        'Indian/Comoro' => 'tempo de Komoroj (Comoro)',
        'Indian/Mahe' => 'tempo de Sejŝeloj (Mahe)',
        'Indian/Maldives' => 'tempo de Maldivoj (Maldives)',
        'Indian/Mauritius' => 'tempo de Maŭricio (Mauritius)',
        'Indian/Mayotte' => 'tempo de Majoto (Mayotte)',
        'Indian/Reunion' => 'tempo de Reunio (Réunion)',
        'Pacific/Apia' => 'tempo de Samoo (Apia)',
        'Pacific/Auckland' => 'tempo de Nov-Zelando (Auckland)',
        'Pacific/Bougainville' => 'tempo de Papuo-Nov-Gvineo (Bougainville)',
        'Pacific/Chatham' => 'tempo de Nov-Zelando (Chatham)',
        'Pacific/Easter' => 'tempo de Ĉilio (Easter)',
        'Pacific/Efate' => 'tempo de Vanuatuo (Efate)',
        'Pacific/Enderbury' => 'tempo de Kiribato (Enderbury)',
        'Pacific/Fiji' => 'tempo de Fiĝoj (Fiji)',
        'Pacific/Funafuti' => 'tempo de Tuvalo (Funafuti)',
        'Pacific/Galapagos' => 'tempo de Ekvadoro (Galapagos)',
        'Pacific/Gambier' => 'tempo de Franca Polinezio (Gambier)',
        'Pacific/Guadalcanal' => 'tempo de Salomonoj (Guadalcanal)',
        'Pacific/Guam' => 'tempo de Gvamo (Guam)',
        'Pacific/Honolulu' => 'tempo de Usono (Honolulu)',
        'Pacific/Kiritimati' => 'tempo de Kiribato (Kiritimati)',
        'Pacific/Kosrae' => 'tempo de Mikronezio (Kosrae)',
        'Pacific/Kwajalein' => 'tempo de Marŝaloj (Kwajalein)',
        'Pacific/Majuro' => 'tempo de Marŝaloj (Majuro)',
        'Pacific/Marquesas' => 'tempo de Franca Polinezio (Marquesas)',
        'Pacific/Midway' => 'tempo de Usonaj malgrandaj insuloj (Midway)',
        'Pacific/Nauru' => 'tempo de Nauro (Nauru)',
        'Pacific/Niue' => 'tempo de Niuo (Niue)',
        'Pacific/Norfolk' => 'tempo de Norfolkinsulo (Norfolk)',
        'Pacific/Noumea' => 'tempo de Nov-Kaledonio (Noumea)',
        'Pacific/Palau' => 'tempo de Palaŭo (Palau)',
        'Pacific/Pitcairn' => 'tempo de Pitkarna Insulo (Pitcairn)',
        'Pacific/Ponape' => 'tempo de Mikronezio (Pohnpei)',
        'Pacific/Port_Moresby' => 'tempo de Papuo-Nov-Gvineo (Port Moresby)',
        'Pacific/Rarotonga' => 'tempo de Kukinsuloj (Rarotonga)',
        'Pacific/Saipan' => 'tempo de Nord-Marianoj (Saipan)',
        'Pacific/Tahiti' => 'tempo de Franca Polinezio (Tahiti)',
        'Pacific/Tarawa' => 'tempo de Kiribato (Tarawa)',
        'Pacific/Tongatapu' => 'tempo de Tongo (Tongatapu)',
        'Pacific/Truk' => 'tempo de Mikronezio (Chuuk)',
        'Pacific/Wake' => 'tempo de Usonaj malgrandaj insuloj (Wake)',
        'Pacific/Wallis' => 'tempo de Valiso kaj Futuno (Wallis)',
    ],
    'Meta' => [
        'GmtFormat' => 'UTC%s',
    ],
];
