<?php

return [
    'Names' => [
        'cv' => 'tsjuvansk',
        'gv' => 'manx',
        'kl' => 'grønlandsk [kalaallisut]',
        'mg' => 'madagassisk',
        'ne' => 'nepalsk',
        'sc' => 'sardinsk',
        'st' => 'sørsotho',
        'tn' => 'tswana',
    ],
];
