<?php

return [
    'Names' => [
        'Arab' => 'erebî',
        'Aran' => 'nestalîq',
        'Armn' => 'ermenî',
        'Beng' => 'bengalî',
        'Bopo' => 'bopomofo',
        'Brai' => 'braille',
        'Copt' => 'qiptî',
        'Cprt' => 'qibrisî',
        'Cyrl' => 'kirîlî',
        'Deva' => 'devanagarî',
        'Egyp' => 'hîyeroglîfên misirî',
        'Ethi' => 'etîyopîk',
        'Geor' => 'gurcî',
        'Goth' => 'gotîk',
        'Grek' => 'yûnanî',
        'Gujr' => 'gujeratî',
        'Hanb' => 'hanîya bi bopomofoyê',
        'Hang' => 'hangulî',
        'Hani' => 'hanî',
        'Hans' => 'sadekirî',
        'Hant' => 'kevneşopî',
        'Hebr' => 'îbranî',
        'Hira' => 'hîraganayî',
        'Hrkt' => 'nivîsên heceyî yên japonî',
        'Jamo' => 'jamoyî',
        'Jpan' => 'japonî',
        'Kana' => 'katakanayî',
        'Khmr' => 'ximêrî',
        'Knda' => 'kannadayî',
        'Kore' => 'koreyî',
        'Laoo' => 'laoyî',
        'Latn' => 'latînî',
        'Mlym' => 'malayamî',
        'Mong' => 'moxolî',
        'Mymr' => 'myanmarî',
        'Qaag' => 'zawgyi',
        'Sinh' => 'sînhalayî',
        'Taml' => 'tamîlî',
        'Telu' => 'teluguyî',
        'Thai' => 'tayî',
        'Tibt' => 'tîbetî',
        'Yezi' => 'êzidî',
        'Zmth' => 'nîşandana matematîkî',
        'Zsye' => 'emojî',
        'Zsym' => 'sembol',
        'Zxxx' => 'nenivîskî',
        'Zyyy' => 'hevpar',
    ],
];
