<?php

return [
    'Names' => [
        'Adlm' => 'Adlam',
        'Arab' => 'Arabies',
        'Aran' => 'Nastaliq',
        'Armn' => 'Armeens',
        'Beng' => 'Bengaals',
        'Bopo' => 'Bopomofo',
        'Brai' => 'Braille',
        'Cakm' => 'Chakma',
        'Cans' => 'Verenigde Kanadese Inheemse Lettergreepskrif',
        'Cher' => 'Cherokee',
        'Cyrl' => 'Cyrillies',
        'Deva' => 'Devanagari',
        'Ethi' => 'Etiopies',
        'Geor' => 'Georgies',
        'Grek' => 'Grieks',
        'Gujr' => 'Goedjarati',
        'Guru' => 'Gurmukhi',
        'Hanb' => 'Han met Bopomofo',
        'Hang' => 'Hangul',
        'Hani' => 'Han',
        'Hans' => 'Vereenvoudig',
        'Hant' => 'Tradisioneel',
        'Hebr' => 'Hebreeus',
        'Hira' => 'Hiragana',
        'Hrkt' => 'Japannese lettergreepskrif',
        'Jpan' => 'Japannees',
        'Kana' => 'Katakana',
        'Khmr' => 'Khmer',
        'Knda' => 'Kannada',
        'Kore' => 'Koreaans',
        'Laoo' => 'Lao',
        'Latn' => 'Latyn',
        'Mlym' => 'Malabaars',
        'Mong' => 'Mongools',
        'Mtei' => 'Meitei-Majek',
        'Mymr' => 'Mianmar',
        'Nkoo' => 'N’Ko',
        'Olck' => 'Ol Chiki',
        'Orya' => 'Oriya',
        'Rohg' => 'Hanifi',
        'Sinh' => 'Sinhala',
        'Sund' => 'Soendanees',
        'Syrc' => 'Siries',
        'Taml' => 'Tamil',
        'Telu' => 'Teloegoe',
        'Tfng' => 'Tifinagh',
        'Thaa' => 'Thaana',
        'Tibt' => 'Tibettaans',
        'Vaii' => 'Vai',
        'Yiii' => 'Yi',
        'Zmth' => 'Wiskundige notasie',
        'Zsye' => 'Emoji',
        'Zsym' => 'Simbole',
        'Zxxx' => 'Ongeskrewe',
        'Zyyy' => 'Algemeen',
    ],
];
