<?php

return [
    'Names' => [
        'Adlm' => 'Adlam',
        'Aghb' => 'Kaukasia Albania',
        'Aran' => 'Nastaliq',
        'Armi' => 'Aramia Imperial',
        'Armn' => 'Armenia',
        'Avst' => 'Avestan',
        'Bali' => 'Bali',
        'Bass' => 'Bassa Vah',
        'Batk' => 'Batak',
        'Beng' => 'Benggala',
        'Bhks' => 'Bhaisuki',
        'Bopo' => 'Bopomofo',
        'Brah' => 'Brahmi',
        'Brai' => 'Braille',
        'Bugi' => 'Bugis',
        'Buhd' => 'Buhid',
        'Cakm' => 'Chakma',
        'Cans' => 'Suku Kata Orang Asli Kanada Bersatu',
        'Cari' => 'Carian',
        'Cham' => 'Cham',
        'Cher' => 'Cherokee',
        'Copt' => 'Coptic',
        'Cprt' => 'Cypriot',
        'Cyrl' => 'Cyril',
        'Deva' => 'Devanagari',
        'Dogr' => 'Dogra',
        'Dsrt' => 'Deseret',
        'Dupl' => 'Trengkas Duployan',
        'Egyp' => 'Hiroglif Mesir',
        'Elba' => 'Elbasan',
        'Elym' => 'Elymaic',
        'Ethi' => 'Ethiopia',
        'Geor' => 'Georgia',
        'Glag' => 'Glagolitik',
        'Gong' => 'Gunjala Gondi',
        'Gonm' => 'Masaram Gonti',
        'Goth' => 'Gothic',
        'Gran' => 'Grantha',
        'Grek' => 'Greek',
        'Gujr' => 'Gujarat',
        'Guru' => 'Gurmukhi',
        'Hanb' => 'Han dengan Bopomofo',
        'Hang' => 'Hangul',
        'Hani' => 'Han',
        'Hano' => 'Hanunoo',
        'Hans' => 'Ringkas',
        'Hant' => 'Tradisional',
        'Hatr' => 'Hatran',
        'Hebr' => 'Ibrani',
        'Hira' => 'Hiragana',
        'Hluw' => 'Hiroglif Anatoli',
        'Hmng' => 'Pahawh Hmong',
        'Hmnp' => 'Nyiakeng Puachue Hmong',
        'Hrkt' => 'Ejaan sukuan Jepun',
        'Hung' => 'Hungary Lama',
        'Ital' => 'Italik Lama',
        'Java' => 'Jawa',
        'Jpan' => 'Jepun',
        'Kali' => 'Kayah Li',
        'Kana' => 'Katakana',
        'Khar' => 'Kharoshthi',
        'Khmr' => 'Khmer',
        'Khoj' => 'Khojki',
        'Knda' => 'Kannada',
        'Kore' => 'Korea',
        'Kthi' => 'Kaithi',
        'Lana' => 'Lanna',
        'Laoo' => 'Lao',
        'Latn' => 'Latin',
        'Lepc' => 'Lepcha',
        'Limb' => 'Limbu',
        'Lina' => 'Linear A',
        'Linb' => 'Linear B',
        'Lisu' => 'Fraser',
        'Lyci' => 'Lycia',
        'Lydi' => 'Lydia',
        'Mahj' => 'Mahajani',
        'Maka' => 'Makasar',
        'Mand' => 'Mandaean',
        'Mani' => 'Manichaean',
        'Marc' => 'Marchen',
        'Medf' => 'Medefaidrin',
        'Mend' => 'Mende',
        'Merc' => 'Kursif Meroitic',
        'Mero' => 'Meroitic',
        'Mlym' => 'Malayalam',
        'Modi' => 'Modi',
        'Mong' => 'Mongolia',
        'Mroo' => 'Mro',
        'Mtei' => 'Meitei Mayek',
        'Mult' => 'Multani',
        'Mymr' => 'Myammar',
        'Nand' => 'Nandinagari',
        'Narb' => 'Arab Utara Lama',
        'Nbat' => 'Nabataean',
        'Nkoo' => 'N’ko',
        'Nshu' => 'Nushu',
        'Ogam' => 'Ogham',
        'Olck' => 'Ol Chiki',
        'Orkh' => 'Orkhon',
        'Orya' => 'Oriya',
        'Osge' => 'Osage',
        'Osma' => 'Osmanya',
        'Palm' => 'Palmyrene',
        'Pauc' => 'Pau Cin Hau',
        'Perm' => 'Permic Lama',
        'Phag' => 'Phags-pa',
        'Phli' => 'Inskripsi Pahlavi',
        'Phlp' => 'Pslater Pahlavi',
        'Phnx' => 'Phoenicia',
        'Plrd' => 'Fonetik Pollard',
        'Prti' => 'Inskripsi Parthian',
        'Qaag' => 'Zawgyi',
        'Rjng' => 'Rejang',
        'Rohg' => 'Hanifi Rohingya',
        'Runr' => 'Runic',
        'Samr' => 'Samaritan',
        'Sarb' => 'Arab Selatan Lama',
        'Saur' => 'Saurashtra',
        'Sgnw' => 'Tulisan Isyarat',
        'Shaw' => 'Shavia',
        'Shrd' => 'Sharada',
        'Sidd' => 'Siddham',
        'Sind' => 'Khudawadi',
        'Sinh' => 'Sinhala',
        'Sogd' => 'Sogdia',
        'Sogo' => 'Sogdia Lama',
        'Sora' => 'Sora Sompeng',
        'Soyo' => 'Soyombo',
        'Sund' => 'Sunda',
        'Sylo' => 'Syloti Nagri',
        'Syrc' => 'Syria',
        'Tagb' => 'Tagbanwa',
        'Takr' => 'Takri',
        'Tale' => 'Tai Le',
        'Talu' => 'Tai Lue Baharu',
        'Taml' => 'Tamil',
        'Tang' => 'Tangut',
        'Tavt' => 'Tai Viet',
        'Telu' => 'Telugu',
        'Tfng' => 'Tifinagh',
        'Tglg' => 'Tagalog',
        'Thaa' => 'Thaana',
        'Tibt' => 'Tibet',
        'Tirh' => 'Tirhuta',
        'Ugar' => 'Ugaritic',
        'Vaii' => 'Vai',
        'Wara' => 'Varang Kshiti',
        'Wcho' => 'Wancho',
        'Xpeo' => 'Parsi Lama',
        'Xsux' => 'Aksara Paku Sumero-Akkadia',
        'Yiii' => 'Yi',
        'Zanb' => 'Segi Empat Zanabazar',
        'Zinh' => 'Diwarisi',
        'Zmth' => 'Tatatanda matematik',
        'Zsye' => 'Emoji',
        'Zsym' => 'Simbol',
        'Zxxx' => 'Tidak ditulis',
        'Zyyy' => 'Lazim',
    ],
];
