<?php

return [
    'Names' => [
        'Adlm' => 'Adlam',
        'Afak' => 'Afaka',
        'Aghb' => 'Albania Kaukasia',
        'Aran' => 'Nastaliq',
        'Armi' => 'Aram Imperial',
        'Armn' => 'Armenia',
        'Avst' => 'Avesta',
        'Bamu' => 'Bamum',
        'Bass' => 'Bassa Vah',
        'Batk' => 'Batak',
        'Beng' => 'Bengali',
        'Bhks' => 'Bhaiksuki',
        'Blis' => 'Blissymbol',
        'Bopo' => 'Bopomofo',
        'Brah' => 'Brahmi',
        'Brai' => 'Braille',
        'Bugi' => 'Bugis',
        'Buhd' => 'Buhid',
        'Cakm' => 'Chakma',
        'Cans' => 'Simbol Aborigin Kanada Kesatuan',
        'Cari' => 'Karia',
        'Cher' => 'Cherokee',
        'Chrs' => 'Chorasmian',
        'Cirt' => 'Cirth',
        'Copt' => 'Koptik',
        'Cpmn' => 'Cypro-Minoan',
        'Cprt' => 'Siprus',
        'Cyrl' => 'Sirilik',
        'Cyrs' => 'Gereja Slavonia Sirilik Lama',
        'Deva' => 'Dewanagari',
        'Diak' => 'Dives Akuru',
        'Dogr' => 'Dogra',
        'Dsrt' => 'Deseret',
        'Dupl' => 'Stenografi Duployan',
        'Egyd' => 'Demotik Mesir',
        'Egyh' => 'Hieratik Mesir',
        'Egyp' => 'Hieroglip Mesir',
        'Elba' => 'Elbasan',
        'Elym' => 'Elymaic',
        'Ethi' => 'Etiopia',
        'Geok' => 'Georgian Khutsuri',
        'Geor' => 'Georgia',
        'Glag' => 'Glagolitic',
        'Gong' => 'Gunjala Gondi',
        'Gonm' => 'Masaram Gondi',
        'Goth' => 'Gothic',
        'Gran' => 'Grantha',
        'Grek' => 'Yunani',
        'Gujr' => 'Gujarat',
        'Guru' => 'Gurmukhi',
        'Hanb' => 'Han dengan Bopomofo',
        'Hang' => 'Hangul',
        'Hani' => 'Han',
        'Hano' => 'Hanunoo',
        'Hans' => 'Sederhana',
        'Hant' => 'Tradisional',
        'Hatr' => 'Hatran',
        'Hebr' => 'Ibrani',
        'Hira' => 'Hiragana',
        'Hluw' => 'Hieroglif Anatolia',
        'Hmng' => 'Pahawh Hmong',
        'Hmnp' => 'Nyiakeng Puachue Hmong',
        'Hrkt' => 'Katakana atau Hiragana',
        'Hung' => 'Hungaria Kuno',
        'Inds' => 'Indus',
        'Ital' => 'Italia Lama',
        'Java' => 'Jawa',
        'Jpan' => 'Jepang',
        'Jurc' => 'Jurchen',
        'Kali' => 'Kayah Li',
        'Kana' => 'Katakana',
        'Khar' => 'Kharoshthi',
        'Khmr' => 'Khmer',
        'Khoj' => 'Khojki',
        'Kits' => 'Skrip kecil Khitan',
        'Knda' => 'Kannada',
        'Kore' => 'Korea',
        'Kpel' => 'Kpelle',
        'Kthi' => 'Kaithi',
        'Lana' => 'Lanna',
        'Laoo' => 'Laos',
        'Latf' => 'Latin Fraktur',
        'Latg' => 'Latin Gaelik',
        'Latn' => 'Latin',
        'Lepc' => 'Lepcha',
        'Limb' => 'Limbu',
        'Lina' => 'Linear A',
        'Linb' => 'Linear B',
        'Lisu' => 'Fraser',
        'Loma' => 'Loma',
        'Lyci' => 'Lycia',
        'Lydi' => 'Lydia',
        'Mahj' => 'Mahajani',
        'Maka' => 'Makassar',
        'Mand' => 'Mandae',
        'Mani' => 'Manikhei',
        'Marc' => 'Marchen',
        'Maya' => 'Hieroglip Maya',
        'Medf' => 'Medefaidrin',
        'Mend' => 'Mende',
        'Merc' => 'Kursif Meroitik',
        'Mero' => 'Meroitik',
        'Mlym' => 'Malayalam',
        'Mong' => 'Mongolia',
        'Moon' => 'Moon',
        'Mroo' => 'Mro',
        'Mtei' => 'Meitei Mayek',
        'Mult' => 'Multani',
        'Mymr' => 'Myanmar',
        'Nand' => 'Nandinagari',
        'Narb' => 'Arab Utara Kuno',
        'Nbat' => 'Nabataea',
        'Nkgb' => 'Naxi Geba',
        'Nkoo' => 'N’Ko',
        'Nshu' => 'Nushu',
        'Ogam' => 'Ogham',
        'Olck' => 'Chiki Lama',
        'Orkh' => 'Orkhon',
        'Orya' => 'Oriya',
        'Osge' => 'Osage',
        'Osma' => 'Osmanya',
        'Ougr' => 'Uyghur Lama',
        'Palm' => 'Palmira',
        'Pauc' => 'Pau Cin Hau',
        'Perm' => 'Permik Kuno',
        'Phag' => 'Phags-pa',
        'Phli' => 'Pahlevi',
        'Phlp' => 'Mazmur Pahlevi',
        'Phlv' => 'Kitab Pahlevi',
        'Phnx' => 'Phoenix',
        'Plrd' => 'Fonetik Pollard',
        'Prti' => 'Prasasti Parthia',
        'Qaag' => 'Zawgyi',
        'Rjng' => 'Rejang',
        'Rohg' => 'Hanifi',
        'Roro' => 'Rongorongo',
        'Runr' => 'Runik',
        'Samr' => 'Samaria',
        'Sara' => 'Sarati',
        'Sarb' => 'Arab Selatan Kuno',
        'Saur' => 'Saurashtra',
        'Sgnw' => 'Tulisan Isyarat',
        'Shaw' => 'Shavia',
        'Shrd' => 'Sharada',
        'Sidd' => 'Siddham',
        'Sind' => 'Khudawadi',
        'Sinh' => 'Sinhala',
        'Sogd' => 'Sogdian',
        'Sogo' => 'Sogdian Lama',
        'Sora' => 'Sora Sompeng',
        'Soyo' => 'Soyombo',
        'Sund' => 'Sunda',
        'Sylo' => 'Syloti Nagri',
        'Syrc' => 'Suriah',
        'Syre' => 'Suriah Estrangelo',
        'Syrj' => 'Suriah Barat',
        'Syrn' => 'Suriah Timur',
        'Tagb' => 'Tagbanwa',
        'Takr' => 'Takri',
        'Tale' => 'Tai Le',
        'Talu' => 'Tai Lue Baru',
        'Taml' => 'Tamil',
        'Tang' => 'Tangut',
        'Tavt' => 'Tai Viet',
        'Telu' => 'Telugu',
        'Teng' => 'Tenghwar',
        'Tfng' => 'Tifinagh',
        'Tglg' => 'Tagalog',
        'Thaa' => 'Thaana',
        'Tibt' => 'Tibet',
        'Tirh' => 'Tirhuta',
        'Tnsa' => 'Tangsa',
        'Toto' => 'Toto (txo)',
        'Ugar' => 'Ugaritik',
        'Vaii' => 'Vai',
        'Visp' => 'Ucapan Terlihat',
        'Vith' => 'Vithkuqi',
        'Wara' => 'Varang Kshiti',
        'Wcho' => 'Wancho',
        'Wole' => 'Woleai',
        'Xpeo' => 'Persia Kuno',
        'Xsux' => 'Cuneiform Sumero-Akkadia',
        'Yezi' => 'Yezidi',
        'Yiii' => 'Yi',
        'Zanb' => 'Zanabazar Square',
        'Zinh' => 'Warisan',
        'Zmth' => 'Notasi Matematika',
        'Zsye' => 'Emoji',
        'Zsym' => 'Simbol',
        'Zxxx' => 'Tidak Tertulis',
        'Zyyy' => 'Umum',
    ],
];
